@@ -4,8 +4,8 @@
  *	  definition of the "relation" system catalog (pg_class)
  *
  *
- * Portions Copyright (c) 2003-2019, PgPool Global Development Group
- * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2003-2021, PgPool Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/catalog/pg_class.h
@@ -24,11 +24,6 @@
 #include "catalog/genbki.h"
 #include "catalog/pg_class_d.h"
 
-/* ----------------
- *		pg_class definition.  cpp turns this into
- *		typedef struct FormData_pg_class
- * ----------------
- */
 CATALOG(pg_class,1259,RelationRelationId) BKI_BOOTSTRAP BKI_ROWTYPE_OID(83,RelationRelation_Rowtype_Id) BKI_SCHEMA_MACRO
 {
 	/* oid */
@@ -38,53 +33,53 @@ CATALOG(pg_class,1259,RelationRelationId) BKI_BOOTSTRAP BKI_ROWTYPE_OID(83,Relat
 	NameData	relname;
 
 	/* OID of namespace containing this class */
-	Oid			relnamespace BKI_DEFAULT(PGNSP);
+	Oid			relnamespace BKI_DEFAULT(pg_catalog) BKI_LOOKUP(pg_namespace);
 
-	/* OID of entry in pg_type for table's implicit row type */
-	Oid			reltype BKI_LOOKUP(pg_type);
+	/* OID of entry in pg_type for relation's implicit row type, if any */
+	Oid			reltype BKI_LOOKUP_OPT(pg_type);
 
-	/* OID of entry in pg_type for underlying composite type */
-	Oid			reloftype BKI_DEFAULT(0) BKI_LOOKUP(pg_type);
+	/* OID of entry in pg_type for underlying composite type, if any */
+	Oid			reloftype BKI_DEFAULT(0) BKI_LOOKUP_OPT(pg_type);
 
 	/* class owner */
-	Oid			relowner BKI_DEFAULT(PGUID);
+	Oid			relowner BKI_DEFAULT(POSTGRES) BKI_LOOKUP(pg_authid);
 
 	/* access method; 0 if not a table / index */
-	Oid			relam BKI_LOOKUP(pg_am);
+	Oid			relam BKI_DEFAULT(heap) BKI_LOOKUP_OPT(pg_am);
 
 	/* identifier of physical storage file */
 	/* relfilenode == 0 means it is a "mapped" relation, see relmapper.c */
-	Oid			relfilenode;
+	Oid			relfilenode BKI_DEFAULT(0);
 
 	/* identifier of table space for relation (0 means default for database) */
-	Oid			reltablespace BKI_DEFAULT(0) BKI_LOOKUP(pg_tablespace);
+	Oid			reltablespace BKI_DEFAULT(0) BKI_LOOKUP_OPT(pg_tablespace);
 
 	/* # of blocks (not always up-to-date) */
-	int32		relpages;
+	int32		relpages BKI_DEFAULT(0);
 
-	/* # of tuples (not always up-to-date) */
-	float4		reltuples;
+	/* # of tuples (not always up-to-date; -1 means "unknown") */
+	float4		reltuples BKI_DEFAULT(-1);
 
 	/* # of all-visible blocks (not always up-to-date) */
-	int32		relallvisible;
+	int32		relallvisible BKI_DEFAULT(0);
 
 	/* OID of toast table; 0 if none */
-	Oid			reltoastrelid;
+	Oid			reltoastrelid BKI_DEFAULT(0) BKI_LOOKUP_OPT(pg_class);
 
 	/* T if has (or has had) any indexes */
-	bool		relhasindex;
+	bool		relhasindex BKI_DEFAULT(f);
 
 	/* T if shared across databases */
-	bool		relisshared;
+	bool		relisshared BKI_DEFAULT(f);
 
 	/* see RELPERSISTENCE_xxx constants below */
-	char		relpersistence;
+	char		relpersistence BKI_DEFAULT(p);
 
 	/* see RELKIND_xxx constants below */
-	char		relkind;
+	char		relkind BKI_DEFAULT(r);
 
 	/* number of user attributes */
-	int16		relnatts;
+	int16		relnatts BKI_DEFAULT(0);	/* genbki.pl will fill this in */
 
 	/*
 	 * Class pg_attribute must contain exactly "relnatts" user attributes
@@ -93,51 +88,51 @@ CATALOG(pg_class,1259,RelationRelationId) BKI_BOOTSTRAP BKI_ROWTYPE_OID(83,Relat
 	 */
 
 	/* # of CHECK constraints for class */
-	int16		relchecks;
+	int16		relchecks BKI_DEFAULT(0);
 
 	/* has (or has had) any rules */
-	bool		relhasrules;
+	bool		relhasrules BKI_DEFAULT(f);
 
 	/* has (or has had) any TRIGGERs */
-	bool		relhastriggers;
+	bool		relhastriggers BKI_DEFAULT(f);
 
 	/* has (or has had) child tables or indexes */
-	bool		relhassubclass;
+	bool		relhassubclass BKI_DEFAULT(f);
 
 	/* row security is enabled or not */
-	bool		relrowsecurity;
+	bool		relrowsecurity BKI_DEFAULT(f);
 
 	/* row security forced for owners or not */
-	bool		relforcerowsecurity;
+	bool		relforcerowsecurity BKI_DEFAULT(f);
 
 	/* matview currently holds query results */
-	bool		relispopulated;
+	bool		relispopulated BKI_DEFAULT(t);
 
 	/* see REPLICA_IDENTITY_xxx constants */
-	char		relreplident;
+	char		relreplident BKI_DEFAULT(n);
 
 	/* is relation a partition? */
-	bool		relispartition;
+	bool		relispartition BKI_DEFAULT(f);
 
-	/* heap for rewrite during DDL, link to original rel */
-	Oid			relrewrite BKI_DEFAULT(0);
+	/* link to original rel during table rewrite; otherwise 0 */
+	Oid			relrewrite BKI_DEFAULT(0) BKI_LOOKUP_OPT(pg_class);
 
 	/* all Xids < this are frozen in this rel */
-	TransactionId relfrozenxid;
+	TransactionId relfrozenxid BKI_DEFAULT(3);	/* FirstNormalTransactionId */
 
 	/* all multixacts in this rel are >= this; it is really a MultiXactId */
-	TransactionId relminmxid;
+	TransactionId relminmxid BKI_DEFAULT(1);	/* FirstMultiXactId */
 
 #ifdef CATALOG_VARLEN			/* variable-length fields start here */
 	/* NOTE: These fields are not present in a relcache entry's rd_rel field. */
 	/* access permissions */
-	aclitem		relacl[1];
+	aclitem		relacl[1] BKI_DEFAULT(_null_);
 
 	/* access-method-specific options */
-	text		reloptions[1];
+	text		reloptions[1] BKI_DEFAULT(_null_);
 
 	/* partition bound node tree */
-	pg_node_tree relpartbound;
+	pg_node_tree relpartbound BKI_DEFAULT(_null_);
 #endif
 } FormData_pg_class;
 
@@ -152,6 +147,12 @@ CATALOG(pg_class,1259,RelationRelationId) BKI_BOOTSTRAP BKI_ROWTYPE_OID(83,Relat
  */
 typedef FormData_pg_class *Form_pg_class;
 
+DECLARE_UNIQUE_INDEX_PKEY(pg_class_oid_index, 2662, on pg_class using btree(oid oid_ops));
+#define ClassOidIndexId  2662
+DECLARE_UNIQUE_INDEX(pg_class_relname_nsp_index, 2663, on pg_class using btree(relname name_ops, relnamespace oid_ops));
+#define ClassNameNspIndexId  2663
+DECLARE_INDEX(pg_class_tblspc_relfilenode_index, 3455, on pg_class using btree(reltablespace oid_ops, relfilenode oid_ops));
+#define ClassTblspcRelfilenodeIndexId  3455
 
 #define                  RELKIND_RELATION                'r'	/* ordinary table */
 #define                  RELKIND_INDEX                   'i'	/* secondary index */
