@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -38,9 +38,16 @@
 #define SM_UNUSED		64
 #define SM_TTY			64
 
+/*
+ * Maximum hostname length including domain name and "." including NULL
+ * terminate.
+ * https://en.wikipedia.org/wiki/Hostname#cite_note-Raymond,_Microsoft_devblog,_2012-3
+ */
+#define MAX_FDQN_HOSTNAME_LEN	254
+
 #define MAX_NUM_BACKENDS 128
 #define MAX_CONNECTION_SLOTS MAX_NUM_BACKENDS
-#define MAX_DB_HOST_NAMELEN	 128
+#define MAX_DB_HOST_NAMELEN	 MAX_FDQN_HOSTNAME_LEN
 #define MAX_PATH_LENGTH 256
 #define NAMEDATALEN 64
 
@@ -69,8 +76,8 @@ typedef struct
 
 typedef enum
 {
-	ROLE_MASTER,
-	ROLE_SLAVE,
+	ROLE_MAIN,
+	ROLE_REPLICA,
 	ROLE_PRIMARY,
 	ROLE_STANDBY
 }			SERVER_ROLE;
@@ -83,11 +90,12 @@ typedef struct
 	char		backend_hostname[MAX_DB_HOST_NAMELEN];	/* backend host name */
 	int			backend_port;	/* backend port numbers */
 	BACKEND_STATUS backend_status;	/* backend status */
+	char		pg_backend_status[NAMEDATALEN];	/* backend status examined by show pool_nodes and pcp_node_info*/
 	time_t		status_changed_time;	/* backend status changed time */
 	double		backend_weight; /* normalized backend load balance ratio */
-	double		unnormalized_weight;	/* descripted parameter */
+	double		unnormalized_weight;	/* described parameter */
 	char		backend_data_directory[MAX_PATH_LENGTH];
-	char		backend_application_name[NAMEDATALEN];	/* application_name for walreciever */
+	char		backend_application_name[NAMEDATALEN];	/* application_name for walreceiver */
 	unsigned short flag;		/* various flags */
 	bool		quarantine;		/* true if node is CON_DOWN because of
 								 * quarantine */
@@ -95,6 +103,7 @@ typedef struct
 	SERVER_ROLE role;			/* Role of server. used by pcp_node_info and
 								 * failover() to keep track of quarantined
 								 * primary node */
+	char		pg_role[NAMEDATALEN];	/* backend role examined by show pool_nodes and pcp_node_info*/
 	char		replication_state [NAMEDATALEN];	/* "state" from pg_stat_replication */
 	char		replication_sync_state [NAMEDATALEN];	/* "sync_state" from pg_stat_replication */
 }			BackendInfo;
@@ -109,6 +118,14 @@ typedef struct
 	BackendInfo backend_info[MAX_NUM_BACKENDS];
 }			BackendDesc;
 
+typedef enum
+{
+	WAIT_FOR_CONNECT,
+	COMMAND_EXECUTE,
+	IDLE,
+	IDLE_IN_TRANS
+}			ProcessStatus;
+
 /*
  * Connection pool information. Placed on shared memory area.
  */
@@ -123,6 +140,9 @@ typedef struct
 	int			key;			/* cancel key */
 	int			counter;		/* used counter */
 	time_t		create_time;	/* connection creation time */
+	time_t		client_connection_time;	/* client connection time */
+	time_t		client_disconnection_time;	/* client last disconnection time */
+	int			client_idle_duration;	/* client idle duration time (s) */
 	int			load_balancing_node;	/* load balancing node */
 	char		connected;		/* True if frontend connected. Please note
 								 * that we use "char" instead of "bool". Since
@@ -149,8 +169,12 @@ typedef struct
 {
 	pid_t		pid;			/* OS's process id */
 	time_t		start_time;		/* fork() time */
+	char		connected;		/* if not 0 this process is already used*/
+	int			wait_for_connect;	/* waiting time for client connection (s) */
 	ConnectionInfo *connection_info;	/* head of the connection info for
 										 * this process */
+	int			client_connection_count;	/* how many times clients used this process */
+	ProcessStatus	status;
 	char		need_to_restart;	/* If non 0, exit this child process as
 									 * soon as current session ends. Typical
 									 * case this flag being set is failback a
@@ -171,7 +195,8 @@ typedef struct
 #define POOLCONFIG_MAXWEIGHTLEN 20
 #define POOLCONFIG_MAXDATELEN 128
 #define POOLCONFIG_MAXCOUNTLEN 16
-
+#define POOLCONFIG_MAXLONGCOUNTLEN 20
+#define POOLCONFIG_MAXPROCESSSTATUSLEN 20
 /* config report struct*/
 typedef struct
 {
@@ -187,8 +212,10 @@ typedef struct
 	char		hostname[MAX_DB_HOST_NAMELEN + 1];
 	char		port[POOLCONFIG_MAXPORTLEN + 1];
 	char		status[POOLCONFIG_MAXSTATLEN + 1];
+	char		pg_status[POOLCONFIG_MAXSTATLEN + 1];
 	char		lb_weight[POOLCONFIG_MAXWEIGHTLEN + 1];
 	char		role[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		pg_role[POOLCONFIG_MAXWEIGHTLEN + 1];
 	char		select[POOLCONFIG_MAXWEIGHTLEN + 1];
 	char		load_balance_node[POOLCONFIG_MAXWEIGHTLEN + 1];
 	char		delay[POOLCONFIG_MAXWEIGHTLEN + 1];
@@ -201,28 +228,35 @@ typedef struct
 typedef struct
 {
 	char		pool_pid[POOLCONFIG_MAXCOUNTLEN + 1];
-	char		start_time[POOLCONFIG_MAXDATELEN + 1];
+	char		process_start_time[POOLCONFIG_MAXDATELEN + 1];
+	char		client_connection_count[POOLCONFIG_MAXCOUNTLEN + 1];
 	char		database[POOLCONFIG_MAXIDENTLEN + 1];
 	char		username[POOLCONFIG_MAXIDENTLEN + 1];
-	char		create_time[POOLCONFIG_MAXDATELEN + 1];
+	char		backend_connection_time[POOLCONFIG_MAXDATELEN + 1];
 	char		pool_counter[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		status[POOLCONFIG_MAXPROCESSSTATUSLEN + 1];
 }			POOL_REPORT_PROCESSES;
 
 /* pools reporting struct */
 typedef struct
 {
-	int			pool_pid;
-	time_t		start_time;
-	int			pool_id;
-	int			backend_id;
+	char		pool_pid[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		process_start_time[POOLCONFIG_MAXDATELEN + 1];
+	char		client_connection_count[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		pool_id[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		backend_id[POOLCONFIG_MAXCOUNTLEN + 1];
 	char		database[POOLCONFIG_MAXIDENTLEN + 1];
 	char		username[POOLCONFIG_MAXIDENTLEN + 1];
-	time_t		create_time;
-	int			pool_majorversion;
-	int			pool_minorversion;
-	int			pool_counter;
-	int			pool_backendpid;
-	int			pool_connected;
+	char		backend_connection_time[POOLCONFIG_MAXDATELEN + 1];
+	char		client_connection_time[POOLCONFIG_MAXDATELEN + 1];
+	char		client_disconnection_time[POOLCONFIG_MAXDATELEN + 1];
+	char		client_idle_duration[POOLCONFIG_MAXDATELEN + 1];
+	char		pool_majorversion[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		pool_minorversion[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		pool_counter[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		pool_backendpid[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		pool_connected[POOLCONFIG_MAXCOUNTLEN + 1];
+	char		status[POOLCONFIG_MAXPROCESSSTATUSLEN + 1];
 }			POOL_REPORT_POOLS;
 
 /* version struct */
@@ -231,6 +265,50 @@ typedef struct
 	char		version[POOLCONFIG_MAXVALLEN + 1];
 }			POOL_REPORT_VERSION;
 
+/* health check statistics report struct */
+typedef struct
+{
+	char		node_id[POOLCONFIG_MAXIDLEN + 1];
+	char		hostname[MAX_DB_HOST_NAMELEN + 1];
+	char		port[POOLCONFIG_MAXPORTLEN + 1];
+	char		status[POOLCONFIG_MAXSTATLEN + 1];
+	char		role[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		last_status_change[POOLCONFIG_MAXDATELEN];
+	char		total_count[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		success_count[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		fail_count[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		skip_count[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		retry_count[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		average_retry_count[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		max_retry_count[POOLCONFIG_MAXCOUNTLEN+1];
+	char		max_health_check_duration[POOLCONFIG_MAXCOUNTLEN+1];
+	char		min_health_check_duration[POOLCONFIG_MAXCOUNTLEN+1];
+	char		average_health_check_duration[POOLCONFIG_MAXLONGCOUNTLEN+1];
+	char		last_health_check[POOLCONFIG_MAXDATELEN];
+	char		last_successful_health_check[POOLCONFIG_MAXDATELEN];
+	char		last_skip_health_check[POOLCONFIG_MAXDATELEN];
+	char		last_failed_health_check[POOLCONFIG_MAXDATELEN];
+}			POOL_HEALTH_CHECK_STATS;
+
+/* show backend statistics report struct */
+typedef struct
+{
+	char		node_id[POOLCONFIG_MAXIDLEN + 1];
+	char		hostname[MAX_DB_HOST_NAMELEN + 1];
+	char		port[POOLCONFIG_MAXPORTLEN + 1];
+	char		status[POOLCONFIG_MAXSTATLEN + 1];
+	char		role[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		select_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		insert_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		update_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		delete_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		ddl_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];
+	char		other_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];	
+	char		panic_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];	
+	char		fatal_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];	
+	char		error_cnt[POOLCONFIG_MAXWEIGHTLEN + 1];	
+}			POOL_BACKEND_STATS;
+
 typedef enum
 {
 	PCP_CONNECTION_OK,
@@ -279,7 +357,7 @@ typedef struct PCPConnInfo
 	char	   *errMsg;			/* error message, or NULL if no error */
 	ConnStateType connState;
 	PCPResultInfo *pcpResInfo;
-	FILE	   *Pfdebug;		/* File pointer to output debug infor */
+	FILE	   *Pfdebug;		/* File pointer to output debug info */
 }			PCPConnInfo;
 
 struct WdInfo;
@@ -287,20 +365,21 @@ struct WdInfo;
 extern PCPConnInfo * pcp_connect(char *hostname, int port, char *username, char *password, FILE *Pfdebug);
 extern void pcp_disconnect(PCPConnInfo * pcpConn);
 
-extern PCPResultInfo * pcp_terminate_pgpool(PCPConnInfo * pcpCon, char mode);
+extern PCPResultInfo * pcp_terminate_pgpool(PCPConnInfo * pcpConn, char mode, char command_scope);
 extern PCPResultInfo * pcp_node_count(PCPConnInfo * pcpCon);
 extern PCPResultInfo * pcp_node_info(PCPConnInfo * pcpCon, int nid);
-
+extern PCPResultInfo * pcp_health_check_stats(PCPConnInfo * pcpCon, int nid);
 extern PCPResultInfo * pcp_process_count(PCPConnInfo * pcpConn);
 extern PCPResultInfo * pcp_process_info(PCPConnInfo * pcpConn, int pid);
+extern PCPResultInfo * pcp_reload_config(PCPConnInfo * pcpConn,char command_scope);
 
 extern PCPResultInfo * pcp_detach_node(PCPConnInfo * pcpConn, int nid);
 extern PCPResultInfo * pcp_detach_node_gracefully(PCPConnInfo * pcpConn, int nid);
 extern PCPResultInfo * pcp_attach_node(PCPConnInfo * pcpConn, int nid);
 extern PCPResultInfo * pcp_pool_status(PCPConnInfo * pcpConn);
 extern PCPResultInfo * pcp_recovery_node(PCPConnInfo * pcpConn, int nid);
-extern PCPResultInfo * pcp_promote_node(PCPConnInfo * pcpConn, int nid);
-extern PCPResultInfo * pcp_promote_node_gracefully(PCPConnInfo * pcpConn, int nid);
+extern PCPResultInfo * pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool promote);
+extern PCPResultInfo * pcp_promote_node_gracefully(PCPConnInfo * pcpConn, int nid, bool promote);
 extern PCPResultInfo * pcp_watchdog_info(PCPConnInfo * pcpConn, int nid);
 extern PCPResultInfo * pcp_set_backend_parameter(PCPConnInfo * pcpConn, char *parameter_name, char *value);
 
@@ -319,6 +398,9 @@ extern int	pcp_result_is_empty(PCPResultInfo * res);
 
 extern char *role_to_str(SERVER_ROLE role);
 
+extern	int * pool_health_check_stats_offsets(int *n);
+extern	int * pool_report_pools_offsets(int *n);
+
 /* ------------------------------
  * pcp_error.c
  * ------------------------------
