@@ -8,7 +8,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -56,7 +56,7 @@ static int	pcp_authorize(PCPConnInfo * pcpConn, char *username, char *password);
 static void pcp_internal_error(PCPConnInfo * pcpConn, const char *fmt,...);
 
 static PCPResultInfo * _pcp_detach_node(PCPConnInfo * pcpConn, int nid, bool gracefully);
-static PCPResultInfo * _pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool gracefully);
+static PCPResultInfo * _pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool gracefully, bool promote);
 static PCPResultInfo * process_pcp_response(PCPConnInfo * pcpConn, char sentMsg);
 static void setCommandSuccessful(PCPConnInfo * pcpConn);
 static void setResultStatus(PCPConnInfo * pcpConn, ResultStateType resultState);
@@ -65,6 +65,7 @@ static int	setNextResultBinaryData(PCPResultInfo * res, void *value, int datalen
 static void setResultIntData(PCPResultInfo * res, unsigned int slotno, int value);
 
 static void process_node_info_response(PCPConnInfo * pcpConn, char *buf, int len);
+static void	process_health_check_stats_response(PCPConnInfo * pcpConn, char *buf, int len);
 static void process_command_complete_response(PCPConnInfo * pcpConn, char *buf, int len);
 static void process_watchdog_info_response(PCPConnInfo * pcpConn, char *buf, int len);
 static void process_process_info_response(PCPConnInfo * pcpConn, char *buf, int len);
@@ -76,7 +77,6 @@ static void process_error_response(PCPConnInfo * pcpConn, char toc, char *buff);
 
 
 static void setResultSlotCount(PCPConnInfo * pcpConn, unsigned int slotCount);
-static void free_processInfo(struct PCPConnInfo *pcpConn, void *ptr);
 static int	PCPFlush(PCPConnInfo * pcpConn);
 
 static bool getPoolPassFilename(char *pgpassfile);
@@ -91,7 +91,7 @@ static char *pwdfMatchesString(char *buf, char *token);
  */
 
 /* Check if PCP connection is connected and authenticated
- * return 1 on successfull 0 otherwise
+ * return 1 on successful 0 otherwise
  */
 
 PCPConnInfo *
@@ -100,7 +100,7 @@ pcp_connect(char *hostname, int port, char *username, char *password, FILE *Pfde
 	struct sockaddr_in addr;
 	struct sockaddr_un unix_addr;
 	struct hostent *hp;
-	char	   *password_fron_file = NULL;
+	char	   *password_from_file = NULL;
 	char		os_user[256];
 	PCPConnInfo *pcpConn = palloc0(sizeof(PCPConnInfo));
 	int			fd;
@@ -227,8 +227,8 @@ pcp_connect(char *hostname, int port, char *username, char *password, FILE *Pfde
 		char		port_str[100];
 
 		snprintf(port_str, sizeof(port_str), "%d", port);
-		password_fron_file = PasswordFromFile(pcpConn, hostname, port_str, username);
-		password = password_fron_file;
+		password_from_file = PasswordFromFile(pcpConn, hostname, port_str, username);
+		password = password_from_file;
 	}
 
 	if (pcp_authorize(pcpConn, username, password) < 0)
@@ -240,8 +240,8 @@ pcp_connect(char *hostname, int port, char *username, char *password, FILE *Pfde
 	else
 		pcpConn->connState = PCP_CONNECTION_OK;
 
-	if (password_fron_file)
-		pfree(password_fron_file);
+	if (password_from_file)
+		pfree(password_from_file);
 
 	return pcpConn;
 }
@@ -365,6 +365,7 @@ static PCPResultInfo * process_pcp_response(PCPConnInfo * pcpConn, char sentMsg)
 			setResultStatus(pcpConn, PCP_RES_ERROR);
 			return pcpConn->pcpResInfo;
 		}
+
 		if (pcp_read(pcpConn->pcpConn, &rsize, sizeof(int)))
 		{
 			pcp_internal_error(pcpConn,
@@ -433,6 +434,13 @@ static PCPResultInfo * process_pcp_response(PCPConnInfo * pcpConn, char sentMsg)
 					process_node_info_response(pcpConn, buf, rsize);
 				break;
 
+			case 'h':
+				if (sentMsg != 'H')
+					setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
+				else
+					process_health_check_stats_response(pcpConn, buf, rsize);
+				break;
+
 			case 'l':
 				if (sentMsg != 'L')
 					setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
@@ -461,6 +469,13 @@ static PCPResultInfo * process_pcp_response(PCPConnInfo * pcpConn, char sentMsg)
 					process_command_complete_response(pcpConn, buf, rsize);
 				break;
 
+			case 'z':
+				if (sentMsg != 'Z')
+					setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
+				else
+					process_command_complete_response(pcpConn, buf, rsize);
+				break;
+
 			case 'w':
 				if (sentMsg != 'W')
 					setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
@@ -599,7 +614,7 @@ pcp_disconnect(PCPConnInfo * pcpConn)
  * --------------------------------
  */
 PCPResultInfo *
-pcp_terminate_pgpool(PCPConnInfo * pcpConn, char mode)
+pcp_terminate_pgpool(PCPConnInfo * pcpConn, char mode, char command_scope)
 {
 	int			wsize;
 
@@ -608,7 +623,10 @@ pcp_terminate_pgpool(PCPConnInfo * pcpConn, char mode)
 		pcp_internal_error(pcpConn, "invalid PCP connection");
 		return NULL;
 	}
-	pcp_write(pcpConn->pcpConn, "T", 1);
+	if (command_scope == 'l')	/*local only*/
+		pcp_write(pcpConn->pcpConn, "T", 1);
+	else
+		pcp_write(pcpConn->pcpConn, "t", 1);
 	wsize = htonl(sizeof(int) + sizeof(char));
 	pcp_write(pcpConn->pcpConn, &wsize, sizeof(int));
 	pcp_write(pcpConn->pcpConn, &mode, sizeof(char));
@@ -679,12 +697,31 @@ pcp_node_count(PCPConnInfo * pcpConn)
 static void
 process_node_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 {
+	char       *index;
 	BackendInfo *backend_info = NULL;
 
-	if (strcmp(buf, "CommandComplete") == 0)
+	if (strcmp(buf, "ArraySize") == 0)
+	{
+		int			ci_size;
+
+		index = (char *) memchr(buf, '\0', len);
+		if (index == NULL)
+			goto INVALID_RESPONSE;
+		index += 1;
+		ci_size = atoi(index);
+
+		setResultStatus(pcpConn, PCP_RES_INCOMPLETE);
+		setResultSlotCount(pcpConn, ci_size);
+		pcpConn->pcpResInfo->nextFillSlot = 0;
+		return;
+	}
+	else if (strcmp(buf, "NodeInfo") == 0)
 	{
 		char	   *index = NULL;
 
+		if (PCPResultStatus(pcpConn->pcpResInfo) != PCP_RES_INCOMPLETE)
+			goto INVALID_RESPONSE;
+
 		backend_info = (BackendInfo *) palloc(sizeof(BackendInfo));
 
 		index = (char *) memchr(buf, '\0', len);
@@ -705,6 +742,18 @@ process_node_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 		index += 1;
 		backend_info->backend_status = atoi(index);
 
+		index = (char *) memchr(index, '\0', len);
+		if (index == NULL)
+			goto INVALID_RESPONSE;
+		index += 1;
+		backend_info->quarantine = atoi(index);
+
+		index = (char *) memchr(index, '\0', len);
+		if (index == NULL)
+			goto INVALID_RESPONSE;
+		index += 1;
+		strlcpy(backend_info->pg_backend_status, index, sizeof(backend_info->pg_backend_status));
+
 		index = (char *) memchr(index, '\0', len);
 		if (index == NULL)
 			goto INVALID_RESPONSE;
@@ -714,10 +763,15 @@ process_node_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 		index = (char *) memchr(index, '\0', len);
 		if (index == NULL)
 			goto INVALID_RESPONSE;
-
 		index++;
 		backend_info->role = atoi(index);
 
+		index = (char *) memchr(index, '\0', len);
+		if (index == NULL)
+			goto INVALID_RESPONSE;
+		index++;
+		strlcpy(backend_info->pg_role, index, sizeof(backend_info->pg_role));
+
 		index = (char *) memchr(index, '\0', len);
 		if (index == NULL)
 			goto INVALID_RESPONSE;
@@ -742,7 +796,7 @@ process_node_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 		index = (char *) memchr(index, '\0', len);
 		if (index == NULL)
 			goto INVALID_RESPONSE;
-		
+
 		index++;
 		backend_info->status_changed_time = atol(index);
 
@@ -753,17 +807,14 @@ process_node_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 		if (setNextResultBinaryData(pcpConn->pcpResInfo, (void *) backend_info, sizeof(BackendInfo), NULL) < 0)
 			goto INVALID_RESPONSE;
 
-		setCommandSuccessful(pcpConn);
+		return;
 	}
-	else
+	else if (strcmp(buf, "CommandComplete") == 0)
 	{
-		pcp_internal_error(pcpConn,
-						   "command failed with reason: \"%s\"", buf);
-		setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
+		setResultStatus(pcpConn, PCP_RES_COMMAND_OK);
+		return;
 	}
 
-	return;
-
 INVALID_RESPONSE:
 
 	if (backend_info)
@@ -807,6 +858,125 @@ pcp_node_info(PCPConnInfo * pcpConn, int nid)
 	return process_pcp_response(pcpConn, 'I');
 }
 
+
+/* --------------------------------
+ * pcp_health_check_stats - get information of health check stats pointed by given argument
+ *
+ * return structure of node information on success, -1 otherwise
+ * --------------------------------
+ */
+PCPResultInfo *
+pcp_health_check_stats(PCPConnInfo * pcpConn, int nid)
+{
+	int			wsize;
+	char		node_id[16];
+
+	if (PCPConnectionStatus(pcpConn) != PCP_CONNECTION_OK)
+	{
+		pcp_internal_error(pcpConn,
+						   "invalid PCP connection");
+		return NULL;
+	}
+
+	snprintf(node_id, sizeof(node_id), "%d", nid);
+
+	pcp_write(pcpConn->pcpConn, "H", 1);
+	wsize = htonl(strlen(node_id) + 1 + sizeof(int));
+	pcp_write(pcpConn->pcpConn, &wsize, sizeof(int));
+	pcp_write(pcpConn->pcpConn, node_id, strlen(node_id) + 1);
+	if (PCPFlush(pcpConn) < 0)
+		return NULL;
+	if (pcpConn->Pfdebug)
+		fprintf(pcpConn->Pfdebug, "DEBUG: send: tos=\"L\", len=%d\n", ntohl(wsize));
+
+	return process_pcp_response(pcpConn, 'H');
+}
+
+PCPResultInfo *
+pcp_reload_config(PCPConnInfo * pcpConn,char command_scope)
+{
+	int                     wsize;
+/*
+ * pcp packet format for pcp_reload_config
+ * z[size][command_scope]
+ */
+	if (PCPConnectionStatus(pcpConn) != PCP_CONNECTION_OK)
+	{
+	   pcp_internal_error(pcpConn, "invalid PCP connection");
+	   return NULL;
+	}
+
+	pcp_write(pcpConn->pcpConn, "Z", 1);
+	wsize = htonl(sizeof(int) + sizeof(char));
+	pcp_write(pcpConn->pcpConn, &wsize, sizeof(int));
+	pcp_write(pcpConn->pcpConn, &command_scope, sizeof(char));
+	if (PCPFlush(pcpConn) < 0)
+	   return NULL;
+	if (pcpConn->Pfdebug)
+	   fprintf(pcpConn->Pfdebug, "DEBUG: send: tos=\"Z\", len=%d\n", ntohl(wsize));
+
+	return process_pcp_response(pcpConn, 'Z');
+}
+
+
+/*
+ * Process health check response from PCP server.
+ * pcpConn: connection to the server
+ * buf:		returned data from server
+ * len:		length of the data
+ */
+static void
+process_health_check_stats_response
+(PCPConnInfo * pcpConn, char *buf, int len)
+{
+	POOL_HEALTH_CHECK_STATS *stats;
+	int		*offsets;
+	int		n;
+	int		i;
+	char	*p;
+	int		maxstr;
+	char	c[] = "CommandComplete";
+
+	if (strcmp(buf, c) != 0)
+	{
+		pcp_internal_error(pcpConn,
+						   "command failed. invalid response");
+		setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
+		return;
+	}
+	buf += sizeof(c);
+
+	/* Allocate health stats memory */
+	stats = palloc0(sizeof(POOL_HEALTH_CHECK_STATS));
+	p = (char *)stats;
+
+	/* Calculate total packet length */
+	offsets = pool_health_check_stats_offsets(&n);
+
+	for (i = 0; i < n; i++)
+	{
+		if (i == n -1)
+			maxstr = sizeof(POOL_HEALTH_CHECK_STATS) - offsets[i];
+		else
+			maxstr = offsets[i + 1] - offsets[i];
+
+		StrNCpy(p + offsets[i], buf, maxstr -1);
+		buf += strlen(buf) + 1;
+	}
+
+	if (setNextResultBinaryData(pcpConn->pcpResInfo, (void *) stats, sizeof(POOL_HEALTH_CHECK_STATS), NULL) < 0)
+	{
+		if (stats)
+			pfree(stats);
+		pcp_internal_error(pcpConn,
+						   "command failed. invalid response");
+		setResultStatus(pcpConn, PCP_RES_BAD_RESPONSE);
+	}
+	else
+		setCommandSuccessful(pcpConn);
+
+}
+
 static void
 process_process_count_response(PCPConnInfo * pcpConn, char *buf, int len)
 {
@@ -893,30 +1063,17 @@ pcp_process_count(PCPConnInfo * pcpConn)
 	return process_pcp_response(pcpConn, 'N');
 }
 
-static void
-free_processInfo(struct PCPConnInfo *pcpConn, void *ptr)
-{
-	ProcessInfo *pi = (ProcessInfo *) ptr;
-
-	if (pcpConn->Pfdebug)
-		fprintf(pcpConn->Pfdebug, "free ProcessInfo structure \n");
-
-	if (pi == NULL)
-	{
-		if (pcpConn->Pfdebug)
-			fprintf(pcpConn->Pfdebug, "ProcessInfo structure is NULL nothing to free \n");
-		return;
-	}
-	if (pi->connection_info)
-		pfree(pi->connection_info);
-	pfree(pi);
-}
-
 static void
 process_process_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 {
 	char	   *index;
-	ProcessInfo *processInfo = NULL;
+	int			*offsets;
+	int			i, n;
+	int			maxstr;
+	char		*p;
+	POOL_REPORT_POOLS	*pools = NULL;
+
+	offsets = pool_report_pools_offsets(&n);
 
 	if (strcmp(buf, "ArraySize") == 0)
 	{
@@ -938,76 +1095,22 @@ process_process_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 		if (PCPResultStatus(pcpConn->pcpResInfo) != PCP_RES_INCOMPLETE)
 			goto INVALID_RESPONSE;
 
-		processInfo = palloc0(sizeof(ProcessInfo));
-		processInfo->connection_info = palloc0(sizeof(ConnectionInfo));
+		pools = palloc0(sizeof(POOL_REPORT_POOLS));
+		p = (char *)pools;
+		buf += strlen(buf) + 1;
 
-		index = (char *) memchr(buf, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->pid = atoi(index);
+		for (i = 0; i < n; i++)
+		{
+			if (i == n -1)
+				maxstr = sizeof(POOL_REPORT_POOLS) - offsets[i];
+			else
+				maxstr = offsets[i + 1] - offsets[i];
 
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		strlcpy(processInfo->connection_info->database, index, SM_DATABASE);
+			StrNCpy(p + offsets[i], buf, maxstr -1);
+			buf += strlen(buf) + 1;
+		}
 
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		strlcpy(processInfo->connection_info->user, index, SM_USER);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->start_time = atol(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->create_time = atol(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->major = atoi(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->minor = atoi(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->counter = atoi(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->backend_id = atoi(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->pid = atoi(index);
-
-		index = (char *) memchr(index, '\0', len);
-		if (index == NULL)
-			goto INVALID_RESPONSE;
-		index += 1;
-		processInfo->connection_info->connected = atoi(index);
-
-		if (setNextResultBinaryData(pcpConn->pcpResInfo, (void *) processInfo, sizeof(ProcessInfo), free_processInfo) < 0)
+		if (setNextResultBinaryData(pcpConn->pcpResInfo, (void *) pools, sizeof(POOL_REPORT_POOLS), NULL) < 0)
 			goto INVALID_RESPONSE;
 
 		return;
@@ -1021,11 +1124,9 @@ process_process_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 
 INVALID_RESPONSE:
 
-	if (processInfo)
+	if (pools)
 	{
-		if (processInfo->connection_info)
-			pfree(processInfo->connection_info);
-		pfree(processInfo);
+		pfree(pools);
 	}
 	pcp_internal_error(pcpConn,
 					   "command failed. invalid response");
@@ -1286,36 +1387,37 @@ pcp_recovery_node(PCPConnInfo * pcpConn, int nid)
 }
 
 /* --------------------------------
- * pcp_promote_node - promote a node given by the argument as new pgpool's master
+ * pcp_promote_node - promote a node given by the argument as new pgpool's main node
  *
  * return 0 on success, -1 otherwise
  * --------------------------------
  */
 PCPResultInfo *
-pcp_promote_node(PCPConnInfo * pcpConn, int nid)
+pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool promote)
 {
-	return _pcp_promote_node(pcpConn, nid, FALSE);
+	return _pcp_promote_node(pcpConn, nid, FALSE, promote);
 }
 
 /* --------------------------------
 
- * and promote a node given by the argument as new pgpool's master
+ * and promote a node given by the argument as new pgpool's main node
  *
  * return 0 on success, -1 otherwise
  * --------------------------------
  */
 PCPResultInfo *
-pcp_promote_node_gracefully(PCPConnInfo * pcpConn, int nid)
+pcp_promote_node_gracefully(PCPConnInfo * pcpConn, int nid, bool switchover)
 {
-	return _pcp_promote_node(pcpConn, nid, TRUE);
+	return _pcp_promote_node(pcpConn, nid, TRUE, switchover);
 }
 
 static PCPResultInfo *
-_pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool gracefully)
+_pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool gracefully, bool switchover)
 {
 	int			wsize;
 	char		node_id[16];
 	char	   *sendchar;
+	char		*switchover_option;	/* n: just change node status, s: switchover primary */
 
 	if (PCPConnectionStatus(pcpConn) != PCP_CONNECTION_OK)
 	{
@@ -1323,17 +1425,31 @@ _pcp_promote_node(PCPConnInfo * pcpConn, int nid, bool gracefully)
 		return NULL;
 	}
 
-	snprintf(node_id, sizeof(node_id), "%d", nid);
+	snprintf(node_id, sizeof(node_id), "%d ", nid);
 
 	if (gracefully)
 		sendchar = "j";
 	else
 		sendchar = "J";
 
+	if (switchover)
+		switchover_option = "s";
+	else
+		switchover_option = "n";
+
 	pcp_write(pcpConn->pcpConn, sendchar, 1);
-	wsize = htonl(strlen(node_id) + 1 + sizeof(int));
+
+	/* caluculate send buffer size */
+	wsize = sizeof(char);	/* protocol. 'j' or 'J' */
+	wsize += strlen(node_id);	/* node id + space */
+	wsize += sizeof(char);	/* promote option */
+	wsize += sizeof(int);	/* buffer length */
+	wsize = htonl(wsize);
+
 	pcp_write(pcpConn->pcpConn, &wsize, sizeof(int));
 	pcp_write(pcpConn->pcpConn, node_id, strlen(node_id) + 1);
+	pcp_write(pcpConn->pcpConn, switchover_option, 1);
+
 	if (PCPFlush(pcpConn) < 0)
 		return NULL;
 	if (pcpConn->Pfdebug)
@@ -1408,6 +1524,15 @@ process_watchdog_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 			json_value_free(root);
 			goto INVALID_RESPONSE;
 		}
+		if (json_get_int_value_for_key(root, "MemberRemoteNodeCount", &wd_cluster_info->memberRemoteNodeCount))
+		{
+			wd_cluster_info->memberRemoteNodeCount = -1;
+		}
+		if (json_get_int_value_for_key(root, "NodesRequireForQuorum", &wd_cluster_info->nodesRequiredForQuorum))
+		{
+			wd_cluster_info->nodesRequiredForQuorum = -1;
+		}
+
 		if (json_get_int_value_for_key(root, "QuorumStatus", &wd_cluster_info->quorumStatus))
 		{
 			json_value_free(root);
@@ -1425,21 +1550,21 @@ process_watchdog_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 		}
 		wd_cluster_info->escalated = tempVal == 0 ? false : true;
 
-		ptr = json_get_string_value_for_key(root, "MasterNodeName");
+		ptr = json_get_string_value_for_key(root, "LeaderNodeName");
 		if (ptr == NULL)
 		{
 			json_value_free(root);
 			goto INVALID_RESPONSE;
 		}
-		strncpy(wd_cluster_info->masterNodeName, ptr, sizeof(wd_cluster_info->masterNodeName) - 1);
+		strncpy(wd_cluster_info->leaderNodeName, ptr, sizeof(wd_cluster_info->leaderNodeName) - 1);
 
-		ptr = json_get_string_value_for_key(root, "MasterHostName");
+		ptr = json_get_string_value_for_key(root, "LeaderHostName");
 		if (ptr == NULL)
 		{
 			json_value_free(root);
 			goto INVALID_RESPONSE;
 		}
-		strncpy(wd_cluster_info->masterHostName, ptr, sizeof(wd_cluster_info->masterHostName) - 1);
+		strncpy(wd_cluster_info->leaderHostName, ptr, sizeof(wd_cluster_info->leaderHostName) - 1);
 
 		/* Get watchdog nodes data */
 		for (i = 0; i < nodeCount; i++)
@@ -1484,6 +1609,22 @@ process_watchdog_info_response(PCPConnInfo * pcpConn, char *buf, int len)
 			}
 			strncpy(wdNodeInfo->delegate_ip, ptr, sizeof(wdNodeInfo->delegate_ip) - 1);
 
+			if (json_get_int_value_for_key(nodeInfoValue, "Membership", &wdNodeInfo->membership_status))
+			{
+				/* would be from the older version. No need to panic */
+				wdNodeInfo->membership_status = 0;
+			}
+
+			ptr = json_get_string_value_for_key(nodeInfoValue, "MembershipString");
+			if (ptr == NULL)
+			{
+				strncpy(wdNodeInfo->membership_status_string, "NOT-Available",
+						sizeof(wdNodeInfo->membership_status_string) - 1);
+			}
+			else
+				strncpy(wdNodeInfo->membership_status_string, ptr,
+						sizeof(wdNodeInfo->membership_status_string) - 1);
+
 			if (json_get_int_value_for_key(nodeInfoValue, "WdPort", &wdNodeInfo->wd_port))
 			{
 				json_value_free(root);
